package com.s3ai;

import com.s3ai.dao.HibernateSessionFactory;
import com.s3ai.entities.Cinema;
import com.s3ai.entities.Ticket;
import com.s3ai.entities.User;
import com.s3ai.services.CinemaService;
import com.s3ai.services.TicketService;
import com.s3ai.services.UserService;
import org.apache.commons.io.IOUtils;
import org.hibernate.Session;
import org.hibernate.Transaction;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import static org.junit.jupiter.api.Assertions.assertNotNull;

public class TestUtils {

    public static void prepareTable(String insertsFile, String table, Boolean cascade) throws IOException {
        var resourceStream = TestUtils.class.getResourceAsStream(insertsFile);
        assertNotNull(resourceStream);
        var insertQueries = IOUtils.toString(resourceStream, StandardCharsets.UTF_8);
        var cascade_str = cascade ? " CASCADE;" : ";";
        Session session = HibernateSessionFactory.getSessionFactory().openSession();
        Transaction transaction = session.beginTransaction();
        session.createSQLQuery("TRUNCATE TABLE " + table + cascade_str).executeUpdate();
        session.createSQLQuery(insertQueries).executeUpdate();
        transaction.commit();
        session.close();
    }

    public static void prepareCinemas() throws IOException {
        prepareTable("/cinemasFill.sql", "cinema", true);
    }

    public static void prepareUsers() throws IOException {
        prepareTable("/usersFill.sql", "cinema_user", true);
    }

    public static void prepareTickets() throws IOException {
        prepareCinemas();
        prepareUsers();
        prepareTable("/ticketsFill.sql", "ticket", false);
    }

    public static <T> T getRandomElement(List<T> elements) {
        var random = new Random();
        return elements.get(random.nextInt(elements.size()));
    }

    public static Cinema getRandomCinema() {
        var random = new Random();
        var cinema = new Cinema();
        List<String> locations = Arrays.asList("Samara", "Izhevsk", "NeoTokyo", "Bangladesh", "Moscow", "Grozny");
        List<String> names = Arrays.asList("Nol", "Tcelkoviy", "Polushka", "Chekushka", "Osmushka", "Pudovichok", "Mediachok", "Silverchok", "Goldenchok", "Deviatichek", "Desatichek");
        var cinemaService = CinemaService.getInstance();
        cinemaService.fillCinemaFields(cinema, names.get(random.nextInt(names.size())), locations.get(random.nextInt(locations.size())), Math.abs(random.nextInt()));
        return cinema;
    }

    private static String randomName() {
        var random = new Random();
        var names = new String[]{"Jane", "Mary", "Paul", "Jason", "Keanu", "Andrew", "Joseph", "Jotaro", "Ivan", "Jolyne", "Walther"};
        var surnames = new String[]{"Doe", "Reeves", "Statham", "Bourne", "Joestar", "Kujoh", "White", "Van Hallen", "Black", "Smith"};
        return String.format("%s %s", names[random.nextInt(names.length)], surnames[random.nextInt(surnames.length)]);
    }

    public static User getRandomUser() {
        var random = new Random();
        var user = new User();
        var userService = UserService.getInstance();
        userService.fillUserFields(user, randomName(), random.nextInt(60) + 10);
        return user;
    }

    private static String randomMovieName() {
        var random = new Random();
        var wordsFirst = new String[]{"Pirates", "Ladies", "Goblins", "Gangsters", "Programmers", "Hackers", "Wrestlers", "Animals", "Guardians"};
        var wordsSecond = new String[]{"of the", "in", "from"};
        var wordsThird = new String[]{"Galaxy", "America", "Russia", "Japan", "Ocean", "Deep", "Caribbean", "Matrix", "Internet", "Woods", "Caves"};
        return String.format("%s %s %s", wordsFirst[random.nextInt(wordsFirst.length)], wordsSecond[random.nextInt(wordsSecond.length)], wordsThird[random.nextInt(wordsThird.length)]);
    }

    public static Ticket getRandomTicket() {
        var random = new Random();
        var ticket = new Ticket();
        var ticketService = TicketService.getInstance();
        var cinemaService = CinemaService.getInstance();
        var userService = UserService.getInstance();
        var users = userService.getAllUsers();
        var cinemas = cinemaService.getAllCinemas();
        ticketService.fillTicketFields(ticket, users.get(random.nextInt(users.size())), cinemas.get(random.nextInt(cinemas.size())), randomMovieName());
        return ticket;
    }
}
