package com.s3ai.services;

import com.s3ai.TestUtils;
import com.s3ai.entities.Cinema;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import javax.persistence.NoResultException;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

class CinemaServiceTest {

    private CinemaService cinemaServiceUnderTest;

    @BeforeEach
    void setUp() throws IOException {
        cinemaServiceUnderTest = CinemaService.getInstance();
        TestUtils.prepareCinemas();
    }

    @Test
    void testGetAllCinemasCount() {
        // Setup
        final int expectedResult = 100;

        // Run the test
        final List<Cinema> result = cinemaServiceUnderTest.getAllCinemas();

        // Verify the results
        assertEquals(expectedResult, result.size());
    }

    @Test
    void testGetCinemaById() {
        // Setup
        final Cinema expectedResult = TestUtils.getRandomElement(cinemaServiceUnderTest.getAllCinemas());

        // Run the test
        final Cinema result = cinemaServiceUnderTest.getCinemaById(expectedResult.getId());

        // Verify the results
        assertEquals(expectedResult, result);
    }

    @Test
    void testSaveCinema() {
        // Setup
        final Cinema cinema = TestUtils.getRandomCinema();

        // Run the test
        cinemaServiceUnderTest.saveCinema(cinema);

        // Verify the results
        assertEquals(cinema, cinemaServiceUnderTest.getCinemaById(cinema.getId()));
    }

    @Test
    void testUpdateCinema() {
        // Setup
        final Cinema cinema = TestUtils.getRandomElement(cinemaServiceUnderTest.getAllCinemas());
        ;
        cinema.setLocation("TESTING");
        cinema.setName("TEST_NAME");
        cinema.setSeatsCount(300);
        // Run the test
        cinemaServiceUnderTest.updateCinema(cinema);

        // Verify the results
        assertEquals("TESTING", cinemaServiceUnderTest.getCinemaById(cinema.getId()).getLocation());
        assertEquals("TEST_NAME", cinemaServiceUnderTest.getCinemaById(cinema.getId()).getName());
        assertEquals(300, cinemaServiceUnderTest.getCinemaById(cinema.getId()).getSeatsCount());
    }

    @Test
    void testDeleteCinema() {
        // Setup
        final Cinema cinema = TestUtils.getRandomElement(cinemaServiceUnderTest.getAllCinemas());

        UUID id = cinema.getId();
        // Run the test
        cinemaServiceUnderTest.deleteCinema(cinema);


        // Verify the results
        assertThrows(NoResultException.class, () -> cinemaServiceUnderTest.getCinemaById(id));
    }

    @Test
    void testFillCinemaFields() {
        // Setup
        final Cinema cinema = new Cinema();

        // Run the test
        cinemaServiceUnderTest.fillCinemaFields(cinema, "name", "location", 0);

        // Verify the results
        assertEquals("location", cinema.getLocation());
        assertEquals("name", cinema.getName());
        assertEquals(0, cinema.getSeatsCount());
    }

    @Test
    void testGetInstance() {
        // Setup
        final CinemaService expectedResult = CinemaService.getInstance();

        // Run the test
        final CinemaService result = CinemaService.getInstance();

        // Verify the results
        assertEquals(expectedResult, result);
    }
}
